﻿Type=Class
Version=2.5
ModulesStructureVersion=1
B4J=true
@EndOfDesignText@
'Class module
#Region  Class Attributes 
	#Event : AddObject_Done(pjob As ParseJob)
	#Event : GetObjects_Done(job As ParseJob)
	#Event : DeleteObject_Done(pjob As ParseJob)
	#Event : SetFile_Done(pjob As ParseJob)
	#Event : GetFile_Done(pjob As ParseJob)
	#Event : GetImage_Done(pjob As ParseJob)
	#Event : DeleteFile_Done(pjob As ParseJob)
	#Event : PushNotification_Done(pjob As ParseJob)
#End Region

Sub Class_Globals
	Private fx As JFX
	Dim updatemap As Map
	Type ParseJob(Success As Boolean, Response As Object, Id As String)
	Dim en As String
End Sub

'Initializes the object, the eventname should be used in the event subs.
Public Sub Initialize(EventName As String)
updatemap.Initialize
DateTime.DateFormat = "yyyy-MM-dd" 
DateTime.TimeFormat = "HH:mm:ss"
en = EventName
End Sub

'Push a notification to all users, with Title and Message.
Public Sub PushSimple(Title As String, Message As String, ParseName As String,TaskId As String)
Dim j As HttpJob
j.Initialize("push"& TaskId, Me)
Dim map1,m1 As Map
Dim l1 As List
l1.Initialize
l1.Add(ParseName)
map1.Initialize
m1.Initialize
m1.Put("alert",Message)
m1.Put("title",Title)
map1.Put("channels",l1)
map1.Put("data", m1)
Dim gen As JSONGenerator
gen.Initialize(map1)
 j.PostString("https://api.parse.com/1/push", gen.ToString)
j.GetRequest.SetHeader("X-Parse-REST-API-Key", Parse.ApiKey)
j.GetRequest.SetHeader("X-Parse-Application-Id", Parse.AppId)
j.GetRequest.SetContentType("application/json") 
End Sub

'Push a notification to all users, with Title and Message.
'The notification will be recieved only by users that are connected to the net until the time UntilTime (expiration time).
Public Sub PushUntil(Title As String, Message As String,ParseName As String, UntilTime As Long, TaskId As String)
Dim st As String
st = DateTime.Date(UntilTime)& "T" & DateTime.Time(UntilTime) & "Z"

Dim j As HttpJob
j.Initialize("push"& TaskId, Me)
Dim map1,m1 As Map
Dim l1 As List
l1.Initialize
l1.Add(ParseName)
map1.Initialize
m1.Initialize
m1.Put("alert",Message)
m1.Put("title",Title)
map1.Put("channels",l1)
map1.Put("expiration_time",st)
map1.Put("data", m1)
Dim gen As JSONGenerator
gen.Initialize(map1)
 j.PostString("https://api.parse.com/1/push", gen.ToString)
j.GetRequest.SetHeader("X-Parse-REST-API-Key", Parse.ApiKey)
j.GetRequest.SetHeader("X-Parse-Application-Id", Parse.AppId)
j.GetRequest.SetContentType("application/json") 
End Sub

'Retrieves object or objects which their value in the key field is as the parameters.
Public Sub GetObjects(key As String, value As Object, TaskId As String)
	Dim j As HttpJob
	j.Initialize("get" & TaskId, Me)
	key = Chr(34) & key & Chr(34)
	Try 
		Dim k As Long = value
		Dim st As String = k
	Catch
		Dim st As String = Chr(34) & value & Chr(34)
	End Try
	j.Download2("https://api.parse.com/1/classes/" & Parse.DBname, Array As String("where", "{" & key & ":" & st & "}"))
	j.GetRequest.SetHeader("X-Parse-REST-API-Key", Parse.ApiKey)
	j.GetRequest.SetHeader("X-Parse-Application-Id", Parse.AppId)
End Sub

'Retrieves object or objects which their value in the key field is as defined by the constraint and the parameters.
'constraint can be one of the following list:
'lt	 - Less Than
'lte -	Less Than OR Equal To
'gt	 -  Greater Than
'gte -	Greater Than OR Equal To
'ne  -	Not Equal To
'In	 -  Contained In
'nin -	Not Contained In
'exists	- A value is set for the key
'Select	- This matches a value for a key In the result of a different query
'dontSelect	- Requires that a key's value not match a value for a key in the result of a different query
'all -	Contains all of the given values
Public Sub GetObjects2(key As String, value As Object, constraint As String, TaskId As String)
	Dim j As HttpJob
	j.Initialize("get" & TaskId, Me)
	key = Chr(34) & key & Chr(34)
	Try 
		Dim k As Long = value
		Dim st As String = k
	Catch
		Dim st As String = Chr(34) & value & Chr(34)
	End Try
	constraint = Chr(34) & "$" & constraint & Chr(34)
    j.Download2("https://api.parse.com/1/classes/Family", Array As String("where", "{"& key &":{" & constraint & ":" & st & "}}"))
	j.GetRequest.SetHeader("X-Parse-REST-API-Key", Parse.ApiKey)
	j.GetRequest.SetHeader("X-Parse-Application-Id", Parse.AppId)
End Sub

'Adds an object with all the values defined in the data map.
'To upload a file or an image to a field use SetFile method after the new object has been added.
Public Sub AddObject(data As Map,  TaskId As String)
Dim j As HttpJob
j.Initialize("new" & TaskId, Me)
Dim gen As JSONGenerator
gen.Initialize(data) 
j.PostString("https://api.parse.com/1/classes/" & Parse.DBname, gen.ToString)
j.GetRequest.SetHeader("X-Parse-REST-API-Key", Parse.ApiKey)
j.GetRequest.SetHeader("X-Parse-Application-Id", Parse.AppId)
j.GetRequest.SetContentType("application/json") 
End Sub

'Updates an object with all the values defined in the data map.
'The selected object is defined by the pair key-value which should be defined so that there is only one object, otherwise only the first will be updated.
'To upload a file or an image to a field use SetFile method.
Public Sub Update(Key As String, value As Object, data As Map, TaskId As String)
	updatemap = data
	Dim j As HttpJob
	j.Initialize("update1" & TaskId , Me)
	Key = Chr(34) & Key & Chr(34)
	Try 
		Dim k As Long = value
		Dim st As String = k
	Catch
		Dim st As String = Chr(34) & value & Chr(34)
	End Try
	j.Download2("https://api.parse.com/1/classes/" & Parse.DBname, Array As String("where", "{" & Key & ":" & st & "}"))
	j.GetRequest.SetHeader("X-Parse-REST-API-Key", Parse.ApiKey)
	j.GetRequest.SetHeader("X-Parse-Application-Id", Parse.AppId)
	' after getting the objectid the putstring operartion is done in jobdone
End Sub

'Gets a file from a field that holds files.
'Example: if "File" is the name of such field,
'retrieve the specific Object using GetObjects method. 
'The content of "File" field is the Map to be used In GetFile method.
'Returns the name of the file.
Public Sub GetFile(filedata As Map, TaskId As String) As String
Dim j As HttpJob
j.Initialize("file" & TaskId  ,Me)
j.Download(filedata.Get("url"))
Dim st As String = filedata.Get("name")
'Dim k As Int = st.LastIndexOf("-")
Return st.SubString(37)
End Sub

'Gets aa image from a field that holds images.
'Example: if "img" is the name of such field,
'retrieve the specific Object using GetObjects method. 
'The content of "img" field is the Map to be used In GetFile method.
'Returns the name of the file.
Public Sub GetImage(filedata As Map, TaskId As String) As String
Dim j As HttpJob
j.Initialize("img" & TaskId  ,Me)
j.Download(filedata.Get("url"))
Dim st As String = filedata.Get("name")
Return st.SubString(37)
End Sub

'Returns the name of the file as appears in the file field.
'Filedata is the map which is stored in the file field.
Public Sub GetFileName(filedata As Map) As String
Dim st As String = filedata.Get("name")
'Dim k As Int = st.LastIndexOf("-")
Return st.SubString(37)
End Sub

'When uploading a file to a field in an Object,
'the file is stored in a parse cloud and a reference to that place is stored in the field.
'The method does all, at the object defined by key-value pair.
'tokey is the name of the field where the reference to the file is.
'Filetype is "text" or "image".
Public Sub SetFile(key As String,value As String,tokey As String, Dir As String, Filename As String,FileType As String, TaskId As String)
	Dim j As HttpJob
	j.Initialize("upload1" & TaskId & "," & key & "," & value & "," & tokey & "," & Dir & "," & Filename & "," & FileType, Me)
	key = Chr(34) & key & Chr(34)
	Try 
		Dim k As Long = value
		Dim st As String = k
	Catch
		Dim st As String = Chr(34) & value & Chr(34)
	End Try
	j.Download2("https://api.parse.com/1/classes/" & Parse.DBname, Array As String("where", "{" & key & ":" & st & "}"))
	j.GetRequest.SetHeader("X-Parse-REST-API-Key", Parse.ApiKey)
	j.GetRequest.SetHeader("X-Parse-Application-Id", Parse.AppId)
End Sub

'Deletes an object.
'If a field contains a File - the File is not deleted from the Parse cloud.
'To delete the file - use DeleteFile prior to DeleteObject.
Public Sub DeleteObject(key As String,value As String, TaskId As String)
' first get the objectid
	Dim j As HttpJob
	j.Initialize("delobj1" & TaskId, Me)
	key = Chr(34) & key & Chr(34)
	Try 
		Dim k As Long = value
		Dim st As String = k
	Catch
		Dim st As String = Chr(34) & value & Chr(34)
	End Try
	j.Download2("https://api.parse.com/1/classes/" & Parse.DBname, Array As String("where", "{" & key & ":" & st & "}"))
	j.GetRequest.SetHeader("X-Parse-REST-API-Key", Parse.ApiKey)
	j.GetRequest.SetHeader("X-Parse-Application-Id", Parse.AppId)
	' the delete operation continues in sub JobDone 
End Sub

'Deletes a file from an object's file field .
'The object is defined by its key-value pair, inkey is the field's name.
'The MasterKey is required to delete a file from the cloud.
Public Sub DeleteFile(key As String,value As String, inkey As String, TaskId As String)
' first get the file name
	If Parse.master <> "" Then
		Dim j As HttpJob
		j.Initialize("delf1" & TaskId & "," & inkey, Me)
		key = Chr(34) & key & Chr(34)
		Try 
			Dim k As Long = value
			Dim st As String = k
		Catch
			Dim st As String = Chr(34) & value & Chr(34)
		End Try
		j.Download2("https://api.parse.com/1/classes/" & Parse.DBname, Array As String("where", "{" & key & ":" & st & "}"))
		j.GetRequest.SetHeader("X-Parse-REST-API-Key", Parse.ApiKey)
		j.GetRequest.SetHeader("X-Parse-Application-Id", Parse.AppId)
		' the delete operation continues in sub JobDone 
	Else
		Dim pjob As ParseJob
		pjob.Initialize
		pjob.Response = "Not permitted to delete file"
		pjob.Success = False
		pjob.Id = TaskId
		If SubExists(Parse.MModule, en & "_DeleteFile_Done") Then
			CallSub2(Parse.MModule,en & "_DeleteFile_Done",pjob)
		End If
	End If
End Sub

Private Sub JobDone(job As HttpJob)
 '*****************************************************************************
	If job.jobname.StartsWith("get") Then
		Dim pjob As ParseJob
		pjob.Initialize
		pjob.Id = job.JobName.SubString(3)
		If job.Success Then
			Dim jp As JSONParser
			jp.Initialize(job.GetString)
			Dim map1 As Map
			map1 = jp.NextObject
			Dim result As List
			result = map1.GetValueAt(0)
			Dim pjob As ParseJob
			pjob.Initialize
			pjob.Response = result
			pjob.Success = True
		Else
			pjob.Response = job.getstring
			pjob.Success = False
		End If
		If SubExists(Parse.MModule, en & "_GetObjects_Done") Then
			CallSub2(Parse.MModule, en & "_GetObjects_Done",pjob)
		End If
	End If
'*****************************************************************************
	If job.jobname.StartsWith("new") Then
		Dim pjob As ParseJob
		pjob.Initialize
		pjob.Response = job.getstring
		pjob.Id = job.JobName.Replace("new","")
		If job.Success Then
			pjob.Success = True
		Else
			pjob.Success = False
		End If
		If SubExists(Parse.MModule, en & "_AddObject_Done") Then
			CallSub2(Parse.MModule, en & "_AddObject_Done",pjob)
		End If
	End If
'*****************************************************************************
	If job.JobName.StartsWith("update1") Then
		If job.Success Then
			Dim objid As String = GetObjectId(job)
			Dim gen As JSONGenerator
			gen.Initialize(updatemap) 
			Dim j As HttpJob
			j.Initialize(job.JobName.Replace("update1","update2"), Me)
			j.PutString("https://api.parse.com/1/classes/Family/" & objid, gen.ToString)
			j.GetRequest.SetHeader("X-Parse-REST-API-Key", Parse.ApiKey)
			j.GetRequest.SetHeader("X-Parse-Application-Id", Parse.AppId)
			j.GetRequest.SetContentType("application/json") 	
		Else
			Dim pjob As ParseJob
			pjob.Initialize
			pjob.Id = job.JobName.Replace("update1","")
			pjob.Success = False
			pjob.Response = "Object not updated"
			If SubExists(Parse.MModule, en & "_Update_Done") Then
				CallSub2(Parse.MModule, en & "_Update_Done",pjob)
			End If
			
		End If
			
	End If
	If job.jobname.StartsWith("update2") Then
		Dim pjob As ParseJob
		pjob.Initialize
		pjob.Id = job.JobName.Replace("update2","")
		If job.Success Then
			pjob.Success = True
			pjob.Response = "Object updated"
		Else
			pjob.Success = False
			pjob.Response = "Object not updated"
		End If
		If SubExists(Parse.MModule, en & "_Update_Done") Then
			CallSub2(Parse.MModule, en & "_Update_Done",pjob)
		End If
	End If

'*****************************************************************************		
	If job.JobName.StartsWith("img") Then
		Dim pjob As ParseJob
		pjob.Initialize
		pjob.Response = job.GetBitmap
		If job.Success Then
			pjob.Success = True
		Else
			pjob.Success = False
		End If
		pjob.Id = job.JobName.SubString(3)
		If SubExists(Parse.MModule, en & "_GetImage_Done") Then
			CallSub2(Parse.MModule, en & "_GetImage_Done",pjob)
		End If
	End If
'*****************************************************************************
	If job.JobName.StartsWith("file") Then
		Dim pjob As ParseJob
		pjob.Initialize
		pjob.Response = job.Getstring
		If job.Success Then
			pjob.Success = True
		Else
			pjob.Success = False
		End If
		pjob.Id = job.JobName.SubString(4)
		If SubExists(Parse.MModule, en & "_GetFile_Done") Then
			CallSub2(Parse.MModule, en & "_GetFile_Done",pjob)
		End If
	End If
'*****************************************************************************	
	If job.JobName.StartsWith("upload1") Then 
		Dim str() As String = Regex.Split(",",job.JobName)
		If job.Success Then
			Dim objid As String = GetObjectId(job)
			Dim j As HttpJob
			j.Initialize(job.JobName.Replace("upload1","upload2") & "," & objid, Me)
			j.PostFile("https://api.parse.com/1/files/" & str(5), str(4) , str(5))
			j.GetRequest.SetHeader("X-Parse-REST-API-Key", Parse.ApiKey)
			j.GetRequest.SetHeader("X-Parse-Application-Id", Parse.AppId)
			If str(6) = "image" Then
				j.GetRequest.SetContentType("image/jpeg") 
			Else
				j.GetRequest.SetContentType("text/plain") 
			End If
		Else
			Dim pjob As ParseJob
			pjob.Initialize
			pjob.Response = job.getstring
			pjob.Success = False
			pjob.Id = str(0).Replace("upload1","")
			If SubExists(Parse.MModule, en & "_SetFile_Done") Then
				CallSub2(Parse.MModule, en & "_SetFile_Done",pjob)
			End If
		End If
	End If	
	If job.JobName.StartsWith("upload2") Then 
		Dim str() As String = Regex.Split(",",job.JobName)
		If job.Success Then	
			Dim j As HttpJob
			j.Initialize(job.JobName.Replace("upload2","upload3"), Me)
			Dim jp As JSONParser
			jp.Initialize(job.GetString)
			Dim map1 As Map
			map1 = jp.NextObject
			Dim fn As String = map1.Get("name")
			Dim m, m1 As Map
			m.Initialize
			m1.Initialize
			m1.Put("name",fn)
			m1.Put("__type","File")
			m.Put(str(3),m1) ' the key is included in the jobname
			Dim gen As JSONGenerator
			gen.Initialize(m)
			j.putString("https://api.parse.com/1/classes/" & Parse.DBname & "/" & str(7), gen.ToString)
			j.GetRequest.SetHeader("X-Parse-REST-API-Key", Parse.ApiKey)
			j.GetRequest.SetHeader("X-Parse-Application-Id", Parse.AppId)
			j.GetRequest.SetContentType("application/json")		
		Else
			Dim pjob As ParseJob
			pjob.Initialize
			pjob.Response = job.getstring
			pjob.Success = False
			pjob.Id = str(0).Replace("upload2","")
			If SubExists(Parse.MModule, en & "_SetFile_Done") Then
				CallSub2(Parse.MModule, en & "_SetFile_Done",pjob)
			End If
		End If
	End If	
	If job.JobName.StartsWith("upload3") Then 
		
		Dim str() As String = Regex.Split(",",job.JobName)
		Dim pjob As ParseJob
		pjob.Initialize
		pjob.Id = str(0).Replace("upload3","")
		If job.Success Then	
			pjob.Success = True
			pjob.Response = "File uploaded"
		Else
			pjob.Success = False
			pjob.Response = "File not uploaded"
		End If
		If SubExists(Parse.MModule, en & "_SetFile_Done") Then
			CallSub2(Parse.MModule, en & "_SetFile_Done",pjob)
		End If
	End If	
'*****************************************************************************		
	If job.JobName.StartsWith("delobj1") Then 
		Dim objid As String = GetObjectId(job)
		Dim j As HttpJob
		j.Initialize(job.JobName.Replace("delobj1","delobj2"), Me)
		j.Delete("https://api.parse.com/1/classes/Family/" & objid)
		j.GetRequest.SetHeader("X-Parse-REST-API-Key", Parse.ApiKey)
		j.GetRequest.SetHeader("X-Parse-Application-Id", Parse.AppId)
	End If
	If job.JobName.StartsWith("delobj2") Then 
		Dim pjob As ParseJob
		pjob.Initialize
		pjob.Response =  "Object deleted"
		If job.Success Then
			pjob.Success = True
		Else
			pjob.Success = False
		End If
		pjob.Id = job.JobName.SubString(8)
		If SubExists(Parse.MModule, en & "_DeleteObject_Done") Then
			CallSub2(Parse.MModule, en & "_DeleteObject_Done",pjob)
		End If
	End If
'*****************************************************************************	
	If job.JobName.StartsWith("delf1") Then 
		If job.Success Then
			Dim jp As JSONParser
			jp.Initialize(job.GetString)
			Dim map1 As Map
			map1 = jp.NextObject
			Dim result As List
			result = map1.GetValueAt(0)
			Dim m As Map = result.Get(0)
			Dim str() As String = Regex.Split(",",job.JobName)
			Dim m1 As Map
			m1 = m.Get(str(1))
			If m1.IsInitialized Then
				Dim filename As String = m1.Get("name") 
				Dim j As HttpJob
				j.Initialize(str(0).Replace("delf1","delf2"), Me)
				j.Delete("https://api.parse.com/1/files/" & filename)
				j.GetRequest.SetHeader("X-Parse-Application-Id", Parse.AppId)
				j.GetRequest.SetHeader( "X-Parse-Master-Key", Parse.master)
			Else
				Dim pjob As ParseJob
				pjob.Initialize
				pjob.Response = job.getstring
				pjob.Id = str(0).Replace("delf1","")
				pjob.Success = False
				If SubExists(Parse.MModule, en & "_DeleteFile_Done") Then
					CallSub2(Parse.MModule, en & "_DeleteFile_Done",pjob)
				End If
			End If
		Else
			Dim pjob As ParseJob
			pjob.Initialize
			pjob.Response = job.getstring
			pjob.Id = str(0).Replace("delf1","")
			pjob.Success = False
			If SubExists(Parse.MModule, en & "_DeleteFile_Done") Then
				CallSub2(Parse.MModule, en & "_DeleteFile_Done",pjob)
			End If
		End If
	End If
	If job.JobName.StartsWith("delf2") Then 
		Dim pjob As ParseJob
		pjob.Initialize
		Dim str() As String = Regex.Split(",",job.JobName)
		pjob.Id = str(0).Replace("delf2", "")
		If job.Success Then
			pjob.Success = True
			pjob.Response = "File deleted"
		Else
			pjob.Success = False
			pjob.Response = "File not deleted"
		End If
		If SubExists(Parse.MModule, en & "_DeleteFile_Done") Then
			CallSub2(Parse.MModule, en & "_DeleteFile_Done",pjob)
		End If
	End If
'*****************************************************************************
	If job.JobName.StartsWith("push") Then
		Dim pjob As ParseJob
		pjob.Initialize
		pjob.Id = job.JobName.Replace("push","")
		If job.Success Then
			pjob.Success = True
			pjob.Response = "Notification pushed"
		Else
			pjob.Success = False
			pjob.Response = "Notification not pushed"
		End If
		If SubExists(Parse.MModule, en & "_PushNotification_Done") Then
			CallSub2(Parse.MModule, en & "_PushNotification_Done",pjob)
		End If
	End If
'*****************************************************************************
'	If job.JobName.StartsWith("put") Then
'		Dim pjob As ParseJob
'		pjob.Initialize
'		pjob.Id = job.JobName.Replace("put","")
'		If job.Success Then
'			pjob.Success = True
'			pjob.Response = "updated"
'		Else
'			pjob.Success = False
'			pjob.Response = "not updated"
'		End If
'		CallSub2(Parse.MModule, "Put_Done" , pjob)
'	End If

	job.Release
End Sub

Private Sub GetObjectId(job As HttpJob) As String
	Dim jp As JSONParser
	jp.Initialize(job.GetString)
	Dim map1 As Map
	map1 = jp.NextObject
	Dim result As List
	result = map1.GetValueAt(0)
	Dim m As Map = result.Get(0)
	Return m.Get("objectId")
End Sub

''Updates an object with all the values defined in the data map.
''The selected object is defined by the pair key-value which should be defined so that there is only one object, otherwise only the first will be updated.
''To upload a file or an image to a field use SetFile method.
'Public Sub Put(Key As String, value As Object, data As Map, TaskId As String)
'	updatemap = data
'	Dim j As HttpJob
'	j.Initialize("put" & TaskId , Me)
'	Dim gen As JSONGenerator
'	Dim m1 As Map
'	m1.Initialize
'	m1.Put(Key,value)
''	data.Put("where",m1)
'	gen.Initialize(data)
'	j.PutString("https://api.parse.com/1/classes/" & DBname,gen.tostring)
'	j.GetRequest.SetHeader("X-Parse-REST-API-Key", ApiKey)
'	j.GetRequest.SetHeader("X-Parse-Application-Id", AppId)
'	' after getting the objectid the putstring operartion is done in jobdone
'End Sub

